package org.greenstone.gsdl3.gs3build.doctypes;

import java.io.File;
import java.net.URL;
import java.net.MalformedURLException;

import java.util.List;
import java.util.Map;
import java.util.ArrayList;

import org.greenstone.gsdl3.gs3build.metadata.METSFile;
import org.greenstone.gsdl3.gs3build.util.*;

public class HTMLDocument extends AbstractDocument
{
  public static final String HTML_DOCUMENT_TYPE = "HTML";

  public HTMLDocument(DocumentID id)
  { super(id);
  }

  /**
   *  Create the HTMLDocument from a given URL - the URL may in fact be a reference
   *  to a local file.
   *
   *  @param <code>URL</code> The location from which to load the file
   */
  public HTMLDocument(URL url)
  { super(url);

    HTMLDoc htmlDoc;
    if (url.toString().startsWith("file://"))
    { htmlDoc = new HTMLDoc(url, url.toString().substring(7));
    }
    else if (url.toString().startsWith("file:/"))
    { htmlDoc = new HTMLDoc(url, url.toString().substring(5));
    }
    else
    { htmlDoc = new HTMLDoc(url);
    }
    
    this._extractDocumentFiles(htmlDoc);
    this._extractDocumentMetadata(htmlDoc);
  }

  private void _extractDocumentMetadata(HTMLDoc htmlDoc)
  { HTMLBlock codedContent = htmlDoc.getCodedContent();
    boolean inTitle = false;
    StringBuffer title = new StringBuffer();

    for (int e = 0; e < codedContent.size(); e ++)
    { if (codedContent.elementAt(e) instanceof HTMLTag)
      {	HTMLTag tag = (HTMLTag) codedContent.elementAt(e);

        if (tag.tagName().equals("meta"))
	{ // check that the name of the metadata item exists
	  String name = tag.idValue("name");
	  if (name == null || name.length() == 0) {
	    continue;
	  }

	  // get the value, if it exists
	  String value = tag.idValue("content");
	  if (value != null && value.length() > 0) {
	    System.out.println("  " + value);
	  }
	  // if value does not exist, default it to being the same
	  // as the name.
	  else {
	    value = name;
	  }

	  this.addDocumentMetadata(name, value);
	}
	else if (tag.tagName().equals("title"))
	{ inTitle = true;
	}
	else if (tag.tagName().equals("/title"))
	{ inTitle = false;
	}
	// cut off when real body content appears - not a perfect
        // implementation, just cheap & cheerful
	else if (tag.tagName().equals("/head"))
	{ break;
	}
	else if (tag.tagName().equals("body"))
	{ break;
	}
      }
      else if (inTitle == true)
      { title.append(codedContent.elementAt(e).toString());
      }
    }
    if (title.length() > 0)
    { this.addDocumentMetadata("title", title.toString());
    }
  }

  private void _extractDocumentFiles(HTMLDoc htmlDoc)
  { URL homeUrl = this.fileSet.getFile(0).getLocation();

    HTMLBlock codedContent = htmlDoc.getCodedContent();
    for (int e = 0; e < codedContent.size(); e ++)
    { if (codedContent.elementAt(e) instanceof HTMLTag)
      { HTMLTag tag = (HTMLTag) codedContent.elementAt(e);

        if (tag.tagName().equals("img"))
	{ String location = tag.idValue("src");

	  try 
	  { // make the url for the image, and then add it to the document list of
	    // 
	    URL imgUrl = new URL(homeUrl, location);
	    METSFile file = this.fileSet.addFile(imgUrl);
	    this.structureSet.getStructure("All").getDivision("All").addFileReference(file.getID());
	  }
	  catch (MalformedURLException ex)
	  { // TODO: report exception/failure to resolve...
	  }
	}
      }
    }
  }

  public String getDocumentType()
  { return HTML_DOCUMENT_TYPE;
  }

  public String getDocumentText()
  { 
    HTMLDoc htmlDoc;
    URL     url =(URL) this.fileSet.getFile(0).getLocation();

    if (url.toString().startsWith("file://"))
    { htmlDoc = new HTMLDoc(url, url.toString().substring(7));
    }
    else if (url.toString().startsWith("file:/"))
    { htmlDoc = new HTMLDoc(url, url.toString().substring(5));
    }
    else
    { htmlDoc = new HTMLDoc(url);
    }
    return htmlDoc.getContent();
  }

  public String getSectionText(String document)
  { return "";
  }
}
