package org.greenstone.gsdl3.gs3build.doctypes;

import java.io.*;
import java.net.*;

import org.greenstone.gsdl3.gs3build.metadata.*;
import org.greenstone.gsdl3.gs3build.util.HTTPTools;

public class TextRecogniser implements RecogniserInterface
{
  DocumentList listRepository;
  
  public TextRecogniser(DocumentList listRepository)
  { this.listRepository = listRepository;
  }

  public boolean parseDocument(METSFile file)
  {
    String MIMEType = file.getMIMEType();
    if (MIMEType == null || 
	MIMEType.equals("text/plain")) {
      URL location = file.getLocation();
      return this.parseDocument(location);
    }
    return false;
  }

  public boolean parseDocument(URL url)
  { String fileName = null;

    if (url.toString().startsWith("file://")) {
      fileName = url.toString().substring(7);
    }
    else if (url.toString().startsWith("file:/")) {
      fileName = url.toString().substring(5);
    }

    if (fileName != null) {
      if (fileName.endsWith(".txt") ||
	  fileName.endsWith(".text"))
      { this.listRepository.addDocument(new TextDocument(url));
	// TODO: spawn knowledge of children too...
        System.out.println(">>> Posting text document " + fileName);
	return true;
      }
    }
    else
    { // Check MIME type
      String mimeType = HTTPTools.getMIMEType(url);

      if (mimeType == "text/plain")
      { System.out.println("Posting Text document " + url.toString());
      
        TextDocument doc = new TextDocument(url);
	this.listRepository.addDocument(doc);
	return true;
      }
    }
		
    return false;
  }
}
